#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .exceptions import ValidationError, SerializerConfigError

BASE_MESSAGE = u'[Row {index}] {verbose_name} {message}'


class BaseField(object):

    def __init__(self, verbose_name, blank=False, default=None):
        self.verbose_name = verbose_name
        self.blank = blank
        self.default = default
        self.value = None
        self.cleaned_value = None

    def validate(self, index):
        if not self.blank and not self.value:
            raise ValidationError(message=BASE_MESSAGE.format(
                index=index,
                verbose_name=self.verbose_name,
                message='is not allow to be blank.')
            )

        if self.value:
            self.data_type_validate(index)
        elif self.default is not None:
            self.cleaned_value = self.default

    def data_type_validate(self, index):
        if type(self.value) is str:
            self.value = self.value.strip()

    def _data_type_validation_helper(self, index, value, data_type, error_message):
        if type(value) is not data_type and not self.blank:
            raise ValidationError(message=BASE_MESSAGE.format(
                index=index,
                verbose_name=self.verbose_name,
                message=error_message
            ))

    def _choice_validation_helper(self, index, value, choices):
        # Check if choices has duplication
        if len(choices) != len(set(choices)):
            raise SerializerConfigError(message='Choice has duplication.')

        if value not in choices:
            choices = u', '.join(choices)
            raise ValidationError(message=BASE_MESSAGE.format(
                index=index,
                verbose_name=self.verbose_name,
                message=u'{} is not correct, it must has one of these {}.'.format(value, choices)
            ))

    def __repr__(self):
        return '<{}- {}>'.format(self.__class__.__name__, str(self))

    def __str__(self):
        return self.value


class DigitBaseField(BaseField):

    def __init__(self, verbose_name, default=None, convert_str=True, blank=False, choices=None):
        super(DigitBaseField, self).__init__(verbose_name, blank)
        self.convert_str = convert_str
        self.default = default
        self.choices = choices


class BaseDateTimeField(BaseField):

    def __init__(self, date_format, date_format_verbose, verbose_name, blank=False):
        super(BaseDateTimeField, self).__init__(verbose_name, blank)
        self.date_format = date_format
        self.date_format_verbose = date_format_verbose



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime
from openpyxl import Workbook


from django_excel_tools import serializers


# Example Usage
class OrderExcelSerializer(serializers.ExcelSerializer):
    QR_SCANNED_CHOICES = (u'有', u'無')

    shop_name = serializers.CharField(max_length=100, verbose_name='Shop Name')
    order_number = serializers.CharField(max_length=100, verbose_name='Order Number')
    quantity = serializers.IntegerField(verbose_name='Quantity')
    sale_datetime = serializers.DateField(
        verbose_name='Sale Datetime', date_format='%Y-%m-%d', date_format_verbose='YYYY-MM-DD'
    )
    inspection_expired_date = serializers.DateField(
        verbose_name='Inspection Expired Date', date_format='%Y%m%d', date_format_verbose='YYYYMMDD'
    )
    registered_date = serializers.DateField(
        verbose_name='Expired Date', date_format='%Y%m', date_format_verbose='YYYYMM'
    )
    weight = serializers.IntegerField(verbose_name='Weight', blank=True, default=0)
    qr_scanned = serializers.CharField(max_length=2, verbose_name='QR Scanned', choices=QR_SCANNED_CHOICES)

    class Meta:
        start_index = 1
        fields = (
            'shop_name',
            'order_number',
            'sale_datetime',
            'quantity',
            'inspection_expired_date',
            'registered_date',
            'weight',
            'qr_scanned'
        )


class WorkbookTesting(object):

    def __init__(self):
        self.workbook = Workbook()
        self.worksheet = self.workbook.active
        self._generate_title()
        self._generate_data()

    def _generate_title(self):
        self.worksheet['A1'] = 'Shop Name'
        self.worksheet['B1'] = 'Order Number'
        self.worksheet['C1'] = 'Sale Date'
        self.worksheet['D1'] = 'Quantity'
        self.worksheet['E1'] = 'Inspection Expired Date'
        self.worksheet['F1'] = 'Registered Date'
        self.worksheet['G1'] = 'Weight'
        self.worksheet['H1'] = 'QR Scanned'

    def _generate_data(self):
        self.worksheet['A2'] = 'Shop A'
        self.worksheet['B2'] = '170707-001-00000-0'
        self.worksheet['C2'] = '2017-07-07'
        self.worksheet['D2'] = 100
        self.worksheet['E2'] = '20180101'
        self.worksheet['F2'] = '201801'
        self.worksheet['G2'] = ''
        self.worksheet['H2'] = 'T'

        self.worksheet['A3'] = 'Shop B'
        self.worksheet['B3'] = '170707-001-00000-1'
        self.worksheet['C3'] = '2017-07-08'
        self.worksheet['D3'] = 1000
        self.worksheet['E3'] = datetime.date(2017, 1, 1)
        self.worksheet['F3'] = '201802'
        self.worksheet['G3'] = '201802'
        self.worksheet['H3'] = u'無'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from openpyxl import Workbook

from django_excel_tools import serializers
from django_excel_tools.exceptions import ColumnNotEqualError, FieldNotExist
from tests.workbook import WorkbookTesting, OrderExcelSerializer


class TestSerializer(unittest.TestCase):

    def setUp(self):
        workbook = Workbook()
        self.worksheet = workbook.active
        self.worksheet['A1'] = 'Field name 1'
        self.worksheet['B2'] = 'Field name 2'
        self.worksheet['A2'] = 'value 1'
        self.worksheet['B2'] = 'value 2'

    def test_should_field_when_no_class_meta(self):
        class Serializer(serializers.ExcelSerializer):
            pass

        with self.assertRaises(AssertionError):
            Serializer(self.worksheet)

    def test_should_failed_when_not_adding_start_index(self):
        class Serializer(serializers.ExcelSerializer):
            class Meta:
                pass
        with self.assertRaises(AssertionError):
            Serializer(self.worksheet)

    def test_should_failed_when_not_adding_fields(self):
        class Serializer(serializers.ExcelSerializer):
            class Meta:
                start_index = 1

        with self.assertRaises(AssertionError):
            Serializer(self.worksheet)

    def test_should_failed_when_no_fields(self):
        class Serializer(serializers.ExcelSerializer):
            class Meta:
                start_index = 1
        with self.assertRaises(AssertionError):
            Serializer(self.worksheet)

    def test_should_failed_when_fields_is_not_set(self):
        class Serializer(serializers.ExcelSerializer):
            field_name_1 = serializers.CharField(max_length=10, verbose_name='Field name 1')
            field_name_2 = serializers.CharField(max_length=10, verbose_name='Field name 2')

            class Meta:
                start_index = 1

        with self.assertRaises(AssertionError):
            Serializer(self.worksheet)

    def test_should_failed_when_field_is_not_defined_included(self):
        class Serializer(serializers.ExcelSerializer):
            field_name_1 = serializers.CharField(max_length=10, verbose_name='Field name 1')

            class Meta:
                start_index = 1
                fields = ('field_name_1', 'field_name')

        with self.assertRaises(FieldNotExist):
            Serializer(self.worksheet)

    def test_should_failed_when_sheet_column_and_field_is_not_match(self):
        class Serializer(serializers.ExcelSerializer):
            field_name_1 = serializers.CharField(max_length=10, verbose_name='Field name 1')

            class Meta:
                start_index = 1
                fields = ('field_name_1',)

        with self.assertRaises(ColumnNotEqualError):
            Serializer(self.worksheet)


class TestValidation(unittest.TestCase):

    def setUp(self):
        workbook = WorkbookTesting()
        self.worksheet = workbook.worksheet

    def test_something(self):
        order_serializer = OrderExcelSerializer(self.worksheet)
        order_serializer.is_valid()
        print(order_serializer.errors)
        print(order_serializer.cleaned_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from collections import OrderedDict

import datetime

import sys

from .exceptions import ValidationError, ColumnNotEqualError, FieldNotExist, ImportOperationFailed, \
    SerializerConfigError
from .fields import BaseField, BASE_MESSAGE, DigitBaseField, BaseDateTimeField


class BaseSerializer(object):

    def __init__(self, worksheet, **kwargs):
        self.kwargs = kwargs
        self._class_meta_validation()

        self.class_fields = [field for field in dir(self)
                             if not (field.startswith("__") or field.startswith("_"))
                             and not callable(getattr(self, field))]
        assert self.class_fields, 'There no fields added to class.'

        self.errors = []
        self.operation_errors = []
        self.cleaned_data = []
        self.fields = OrderedDict()
        self.worksheet = worksheet

        self._set_fields()
        self._validate_column()

        if not self.errors:
            self._set_values()
            self._start_operation()
        else:
            self.invalid(self.errors)

    def _class_meta_validation(self):
        assert hasattr(self, 'Meta'), 'class Meta is required'

        assert hasattr(self.Meta, 'start_index'), 'Meta.start_index in class Meta is required.'
        assert type(self.Meta.start_index) is int, 'Meta.start_index type is int.'

        assert hasattr(self.Meta, 'fields'), 'Meta.fields in class Meta is required'
        fields_is_list_or_tuple = type(self.Meta.fields) is list or type(self.Meta.fields) is tuple
        assert fields_is_list_or_tuple, 'Meta.fields type must be list or tuple'

        self.field_names = self.Meta.fields
        self.start_index = self.Meta.start_index

    def _set_fields(self):
        for field_name in self.field_names:
            try:
                self.fields[field_name] = getattr(self, field_name)
            except AttributeError:
                raise FieldNotExist(message='{} is not defined in class field.'.format(field_name))

    def _validate_column(self):
        if self.worksheet.max_column != len(self.fields):
            raise ColumnNotEqualError(message='{} has {} fields while sheet has {}, field should be the same.'.format(
                self.__class__.__name__,
                len(self.fields),
                self.worksheet.max_column,
            ))

    def _set_values(self):
        for row_index, row in enumerate(self.worksheet.rows):
            if row_index < self.start_index:
                continue

            for index, cell in enumerate(row):
                key = self.field_names[index]
                self.fields[key].value = cell.value
                try:
                    self.fields[key].validate(index=row_index + 1)
                except ValidationError as error:
                    self.errors.append(error.message)

            self._set_cleaned_values(self.fields)

    def _set_cleaned_values(self, validated_fields):
        cleaned_row = {}
        for key in validated_fields:
            cleaned_value = validated_fields[key].cleaned_value
            try:
                extra_clean = 'extra_clean_{}'.format(key)
                extra_clean_def = getattr(self, extra_clean)
                if callable(extra_clean_def):
                    cleaned_value = extra_clean_def(cleaned_value)
            except AttributeError:
                pass
            cleaned_row[key] = cleaned_value
        self.cleaned_data.append(cleaned_row)

    def _start_operation(self):
        try:
            from django.db import transaction
        except ImportError:
            raise SerializerConfigError(message='Django is required, please make sure you installed Django via pip.')

        try:
            with transaction.atomic():
                self.import_operation(self.cleaned_data)
            self.operation_success()
        except ImportOperationFailed:
            self.operation_failed(self.operation_errors)

    def import_operation(self, cleaned_data):
        pass

    def invalid(self, errors):
        pass

    def operation_failed(self, errors):
        pass

    def operation_success(self):
        pass


class ExcelSerializer(BaseSerializer):

    def gen_error(self, index, error):
        sheet_index = self.start_index + index
        return '[Row {sheet_index} - {error}]'.format(sheet_index=sheet_index, error=error)


class CharField(BaseField):

    def __init__(self, max_length, verbose_name, convert_number=True, blank=False, choices=None, default=None):
        super(CharField, self).__init__(verbose_name, blank)
        self.max_length = max_length
        self.convert_number = convert_number
        self.choices = choices
        self.default = default

    def data_type_validate(self, index):
        super(CharField, self).data_type_validate(index)

        value = self.value
        if self.convert_number:
            if sys.version_info >= (3, 0):
                value = str(value).strip()
            else:
                value = unicode(value).strip()

        type_error_message = BASE_MESSAGE.format(
            index=index,
            verbose_name=self.verbose_name,
            message='must be text.'
        )

        if sys.version_info >= (3, 0):
            if type(value) is not str:
                raise ValidationError(message=type_error_message)
        else:
            if type(value) is not str and type(value) is not unicode:
                raise ValidationError(message=type_error_message)

        if len(value) > self.max_length:
            raise ValidationError(message=BASE_MESSAGE.format(
                index=index,
                verbose_name=self.verbose_name,
                message='cannot be more than {} characters.'.format(self.max_length)
            ))

        if self.choices:
            self._choice_validation_helper(index, value, self.choices)

        if self.default and value is None:
            value = self.default

        self.cleaned_value = value


class IntegerField(DigitBaseField):

    def data_type_validate(self, index):
        super(IntegerField, self).data_type_validate(index)

        value = self.value
        if self.convert_str and type(value) is not int:
            try:
                value = int(value)
            except ValueError:
                raise ValidationError(message=BASE_MESSAGE.format(
                    index=index,
                    verbose_name=self.verbose_name,
                    message='cannot convert {} to number.'.format(value)
                ))

        self._data_type_validation_helper(
            index=index,
            value=value,
            data_type=int,
            error_message='expected type is number but received {}.'.format(type(value).__name__)
        )

        if self.choices:
            self._choice_validation_helper(index, value, self.choices)

        if self.default and value is None:
            value = self.default

        self.cleaned_value = value


class DateField(BaseDateTimeField):

    def data_type_validate(self, index):
        super(DateField, self).data_type_validate(index)

        value = self.value
        type_error_message = BASE_MESSAGE.format(
                    index=index,
                    verbose_name=self.verbose_name,
                    message='"{}" is incorrect format, it should be "{}".'.format(value, self.date_format_verbose)
                )

        def convert_date():
            try:
                return datetime.datetime.strptime(value, self.date_format).date()
            except ValueError:
                raise ValidationError(message=type_error_message)

        if sys.version_info >= (3, 0):
            if type(value) is str:
                value = convert_date()
        else:
            if type(value) is str or type(value) is unicode:
                value = convert_date()

        if type(value) is int:
            value = str(value)
            value = convert_date()

        if type(value) is datetime.datetime:
            value = value.date()

        self._data_type_validation_helper(
            index=index,
            value=value,
            data_type=datetime.date,
            error_message='expected type is date but received {}.'.format(type(value).__name__)
        )

        self.cleaned_value = value


class DateTimeField(BaseDateTimeField):

    def data_type_validate(self, index):
        super(DateTimeField, self).data_type_validate(index)

        value = self.value
        type_error_message = BASE_MESSAGE.format(
                        index=index,
                        verbose_name=self.verbose_name,
                        message='"{}" is incorrect format, it should be "{}"'.format(value, self.date_format_verbose)
                    )

        def convert_date_time():
            try:
                return datetime.datetime.strptime(value, self.date_format)
            except ValueError:
                raise ValidationError(message=type_error_message)

        if sys.version_info >= (3, 0):
            if type(value) is str:
                value = convert_date_time()
        else:
            if type(value) is str or type(value) is unicode:
                value = convert_date_time()

        if type(value) is int:
            value = str(value)
            value = convert_date_time()

        self._data_type_validation_helper(
            index=index,
            value=value,
            data_type=datetime.datetime,
            error_message='expected type is datetime but received {}.'.format(type(value).__name__)
        )

        self.cleaned_value = value



class BaseExcelError(Exception):

    def __init__(self, message):
        super(BaseExcelError, self).__init__()
        self.message = message


class ColumnNotEqualError(BaseExcelError):
    pass


class FieldNotExist(BaseExcelError):
    pass


class ImportOperationFailed(Exception):
    pass


class SerializerConfigError(BaseExcelError):
    pass


class ValidationError(BaseExcelError):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

setup(
    name='django_excel_tools',
    version='0.0.4',
    description="Common function when working with excel.",
    long_description=readme + '\n\n' + history,
    author="Khemanorak Khath",
    author_email='khath.khemanorak@gmail.com',
    url='https://github.com/NorakGithub/django_excel_tools',
    packages=find_packages(include=['django_excel_tools']),
    include_package_data=True,
    license="MIT license",
    zip_safe=False,
    keywords='django, excel, tools',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ]
)
